"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _assertThisInitialized2 = _interopRequireDefault(require("@babel/runtime/helpers/assertThisInitialized"));

var _inheritsLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/inheritsLoose"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _react = _interopRequireDefault(require("react"));

var _warning = _interopRequireDefault(require("warning"));

var _isPromise = _interopRequireDefault(require("is-promise"));

var _TreeContext = _interopRequireDefault(require("./TreeContext"));

var _Node = _interopRequireDefault(require("./Node"));

var TreeNode = /*#__PURE__*/function (_React$Component) {
  (0, _inheritsLoose2.default)(TreeNode, _React$Component);

  function TreeNode(props, context) {
    var _this;

    _this = _React$Component.call(this, props, context) || this;
    (0, _defineProperty2.default)((0, _assertThisInitialized2.default)(_this), "context", void 0);
    _this.state = {
      isLoading: false
    };
    return _this;
  }

  var _proto = TreeNode.prototype;

  _proto.getTree = function getTree() {
    return this.context.tree;
  };

  _proto.getTreeProps = function getTreeProps() {
    var tree = this.getTree();
    return tree.props;
  };

  _proto.getNodeList = function getNodeList(list) {
    var tree = this.getTree();
    var pNode = this.props.node;
    var options = this.getTreeProps();
    var state = tree.state;
    return list.map(function (data, i) {
      var node = new _Node.default(data, pNode, options, state);
      return /*#__PURE__*/_react.default.createElement(TreeNode, {
        render: options.nodeRender,
        node: node,
        key: node.getId()
      });
    });
  };

  _proto.loadNodeChildren = function loadNodeChildren() {
    var _this2 = this;

    var _this$props = this.props,
        node = _this$props.node,
        isRoot = _this$props.isRoot;
    var isLoading = this.state.isLoading;

    var _this$getTreeProps = this.getTreeProps(),
        loadData = _this$getTreeProps.loadData,
        asyncTestDelay = _this$getTreeProps.asyncTestDelay,
        loadRender = _this$getTreeProps.loadRender;

    if (isLoading) {
      return loadRender ? loadRender({
        root: isRoot,
        node: node,
        data: node.data
      }) : null;
    }

    var asyncTimer = null;

    var onfulfilled = function onfulfilled() {
      if (asyncTimer) {
        clearTimeout(asyncTimer);
        asyncTimer = null;
      }

      node.setLoading(false);

      _this2.setState({
        isLoading: false
      });
    };

    var onrejected = onfulfilled;
    var results = loadData(node.data, node);

    if ((0, _isPromise.default)(results)) {
      asyncTimer = setTimeout(function () {
        asyncTimer = null;
        node.setLoading(true);

        _this2.setState({
          isLoading: true
        });
      }, asyncTestDelay);
      results.then(onfulfilled, onrejected);
      return null;
    } else {
      return this.getNodeList(results);
    }
  };

  _proto.renderNodeChildren = function renderNodeChildren() {
    var _this3 = this;

    var _this$getTreeProps2 = this.getTreeProps(),
        maxDepth = _this$getTreeProps2.maxDepth,
        childrenRender = _this$getTreeProps2.childrenRender;

    var _this$props2 = this.props,
        node = _this$props2.node,
        isRoot = _this$props2.isRoot;
    var leaf = node.isLeaf();
    var expanded = !leaf && node.isExpanded();

    if (expanded && node.getDepth() >= maxDepth) {
      process.env.NODE_ENV !== "production" ? (0, _warning.default)(false, "react-widget-tree-basic: maximum depth: " + maxDepth) : void 0;
      return null;
    }

    return leaf ? null : childrenRender({
      getChildren: function getChildren() {
        return _this3.loadNodeChildren();
      },
      loading: node.isLoading(),
      root: isRoot,
      expanded: expanded,
      node: node,
      data: node.data
    });
  };

  _proto.render = function render() {
    var _this4 = this;

    var _this$props3 = this.props,
        node = _this$props3.node,
        isRoot = _this$props3.isRoot,
        render = _this$props3.render;

    var _this$getTreeProps3 = this.getTreeProps(),
        itemRender = _this$getTreeProps3.itemRender,
        childrenRender = _this$getTreeProps3.childrenRender;

    if (isRoot) {
      return childrenRender({
        getChildren: function getChildren() {
          return _this4.loadNodeChildren();
        },
        loading: node.isLoading(),
        root: true,
        expanded: true,
        node: node,
        data: node.data
      });
    }

    var item = itemRender({
      node: node,
      loading: node.isLoading(),
      leaf: node.isLeaf(),
      data: node.getData(),
      expanded: node.isExpanded()
    });
    return render ? render(this.props, item, this.renderNodeChildren()) : /*#__PURE__*/_react.default.createElement(_react.default.Fragment, null, item, this.renderNodeChildren());
  };

  return TreeNode;
}(_react.default.Component);

(0, _defineProperty2.default)(TreeNode, "defaultProps", {
  node: {},
  isRoot: false
});
(0, _defineProperty2.default)(TreeNode, "contextType", _TreeContext.default);
var _default = TreeNode;
exports.default = _default;