import _assertThisInitialized from "@babel/runtime/helpers/assertThisInitialized";
import _inheritsLoose from "@babel/runtime/helpers/inheritsLoose";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
import React from "react";
import warning from "warning";
import isPromise from "is-promise";
import TreeContext from "./TreeContext";
import Node from "./Node";

var TreeNode = /*#__PURE__*/function (_React$Component) {
  _inheritsLoose(TreeNode, _React$Component);

  function TreeNode(props, context) {
    var _this;

    _this = _React$Component.call(this, props, context) || this;

    _defineProperty(_assertThisInitialized(_this), "context", void 0);

    _this.state = {
      isLoading: false
    };
    return _this;
  }

  var _proto = TreeNode.prototype;

  _proto.getTree = function getTree() {
    return this.context.tree;
  };

  _proto.getTreeProps = function getTreeProps() {
    var tree = this.getTree();
    return tree.props;
  };

  _proto.getNodeList = function getNodeList(list) {
    var tree = this.getTree();
    var pNode = this.props.node;
    var options = this.getTreeProps();
    var state = tree.state;
    return list.map(function (data, i) {
      var node = new Node(data, pNode, options, state);
      return /*#__PURE__*/React.createElement(TreeNode, {
        render: options.nodeRender,
        node: node,
        key: node.getId()
      });
    });
  };

  _proto.loadNodeChildren = function loadNodeChildren() {
    var _this2 = this;

    var _this$props = this.props,
        node = _this$props.node,
        isRoot = _this$props.isRoot;
    var isLoading = this.state.isLoading;

    var _this$getTreeProps = this.getTreeProps(),
        loadData = _this$getTreeProps.loadData,
        asyncTestDelay = _this$getTreeProps.asyncTestDelay,
        loadRender = _this$getTreeProps.loadRender;

    if (isLoading) {
      return loadRender ? loadRender({
        root: isRoot,
        node: node,
        data: node.data
      }) : null;
    }

    var asyncTimer = null;

    var onfulfilled = function onfulfilled() {
      if (asyncTimer) {
        clearTimeout(asyncTimer);
        asyncTimer = null;
      }

      node.setLoading(false);

      _this2.setState({
        isLoading: false
      });
    };

    var onrejected = onfulfilled;
    var results = loadData(node.data, node);

    if (isPromise(results)) {
      asyncTimer = setTimeout(function () {
        asyncTimer = null;
        node.setLoading(true);

        _this2.setState({
          isLoading: true
        });
      }, asyncTestDelay);
      results.then(onfulfilled, onrejected);
      return null;
    } else {
      return this.getNodeList(results);
    }
  };

  _proto.renderNodeChildren = function renderNodeChildren() {
    var _this3 = this;

    var _this$getTreeProps2 = this.getTreeProps(),
        maxDepth = _this$getTreeProps2.maxDepth,
        childrenRender = _this$getTreeProps2.childrenRender;

    var _this$props2 = this.props,
        node = _this$props2.node,
        isRoot = _this$props2.isRoot;
    var leaf = node.isLeaf();
    var expanded = !leaf && node.isExpanded();

    if (expanded && node.getDepth() >= maxDepth) {
      process.env.NODE_ENV !== "production" ? warning(false, "react-widget-tree-basic: maximum depth: " + maxDepth) : void 0;
      return null;
    }

    return leaf ? null : childrenRender({
      getChildren: function getChildren() {
        return _this3.loadNodeChildren();
      },
      loading: node.isLoading(),
      root: isRoot,
      expanded: expanded,
      node: node,
      data: node.data
    });
  };

  _proto.render = function render() {
    var _this4 = this;

    var _this$props3 = this.props,
        node = _this$props3.node,
        isRoot = _this$props3.isRoot,
        render = _this$props3.render;

    var _this$getTreeProps3 = this.getTreeProps(),
        itemRender = _this$getTreeProps3.itemRender,
        childrenRender = _this$getTreeProps3.childrenRender;

    if (isRoot) {
      return childrenRender({
        getChildren: function getChildren() {
          return _this4.loadNodeChildren();
        },
        loading: node.isLoading(),
        root: true,
        expanded: true,
        node: node,
        data: node.data
      });
    }

    var item = itemRender({
      node: node,
      loading: node.isLoading(),
      leaf: node.isLeaf(),
      data: node.getData(),
      expanded: node.isExpanded()
    });
    return render ? render(this.props, item, this.renderNodeChildren()) : /*#__PURE__*/React.createElement(React.Fragment, null, item, this.renderNodeChildren());
  };

  return TreeNode;
}(React.Component);

_defineProperty(TreeNode, "defaultProps", {
  node: {},
  isRoot: false
});

_defineProperty(TreeNode, "contextType", TreeContext);

export default TreeNode;