import React from "react";
import warning from "warning";
import isPromise from "is-promise";
import TreeContext from "./TreeContext";
import Node from "./Node";
class TreeNode extends React.Component {
    constructor(props, context) {
        super(props, context);
        this.state = {
            isLoading: false,
        };
    }
    getTree() {
        return this.context.tree;
    }
    getTreeProps() {
        const tree = this.getTree();
        return tree.props;
    }
    getNodeList(list) {
        const tree = this.getTree();
        const { node: pNode } = this.props;
        const options = this.getTreeProps();
        const state = tree.state;
        return list.map((data, i) => {
            const node = new Node(data, pNode, options, state);
            return React.createElement(TreeNode, { render: options.nodeRender, node: node, key: node.getId() });
        });
    }
    loadNodeChildren() {
        const { node, isRoot } = this.props;
        const { isLoading } = this.state;
        const { loadData, asyncTestDelay, loadRender } = this.getTreeProps();
        if (isLoading) {
            return loadRender
                ? loadRender({
                    root: isRoot,
                    node: node,
                    data: node.data,
                })
                : null;
        }
        let asyncTimer = null;
        const onfulfilled = () => {
            if (asyncTimer) {
                clearTimeout(asyncTimer);
                asyncTimer = null;
            }
            node.setLoading(false);
            this.setState({
                isLoading: false,
            });
        };
        const onrejected = onfulfilled;
        const results = loadData(node.data, node);
        if (isPromise(results)) {
            asyncTimer = setTimeout(() => {
                asyncTimer = null;
                node.setLoading(true);
                this.setState({
                    isLoading: true,
                });
            }, asyncTestDelay);
            results.then(onfulfilled, onrejected);
            return null;
        }
        else {
            return this.getNodeList(results);
        }
    }
    renderNodeChildren() {
        const { maxDepth, childrenRender } = this.getTreeProps();
        const { node, isRoot } = this.props;
        const leaf = node.isLeaf();
        const expanded = !leaf && node.isExpanded();
        if (expanded && node.getDepth() >= maxDepth) {
            warning(false, `react-widget-tree-basic: maximum depth: ${maxDepth}`);
            return null;
        }
        return leaf
            ? null
            : childrenRender({
                getChildren: () => this.loadNodeChildren(),
                loading: node.isLoading(),
                root: isRoot,
                expanded,
                node: node,
                data: node.data,
            });
    }
    render() {
        const { node, isRoot, render } = this.props;
        const { itemRender, childrenRender } = this.getTreeProps();
        if (isRoot) {
            return childrenRender({
                getChildren: () => this.loadNodeChildren(),
                loading: node.isLoading(),
                root: true,
                expanded: true,
                node: node,
                data: node.data,
            });
        }
        const item = itemRender({
            node,
            loading: node.isLoading(),
            leaf: node.isLeaf(),
            data: node.getData(),
            expanded: node.isExpanded(),
        });
        return render ? (render(this.props, item, this.renderNodeChildren())) : (React.createElement(React.Fragment, null,
            item,
            this.renderNodeChildren()));
    }
}
TreeNode.defaultProps = {
    node: {},
    isRoot: false,
};
TreeNode.contextType = TreeContext;
export default TreeNode;
