import React from "react";
import classNames from "classnames";
import TreeContext from "./TreeContext";
import Node from "./Node";
import TreeNode from "./TreeNode";
export const version = "%VERSION%";
export function toMarked(array) {
    const marked = Object.create(null);
    array.forEach((value) => {
        marked[value] = true;
    });
    return marked;
}
export class TreeBasic extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            expandedIds: [],
            expandedMap: {},
        };
    }
    static getDerivedStateFromProps(nextProps) {
        return {
            expandedIds: nextProps.expandedIds,
            expandedMap: toMarked(nextProps.expandedIds || []),
        };
    }
    getRootNode() {
        const { rootId, idField, pidField, leafField } = this.props;
        const node = new Node({
            [idField]: rootId,
            [pidField]: null,
            [leafField]: false,
        }, null, this.props, this.state);
        return node;
    }
    getContext() {
        return {
            tree: this,
        };
    }
    render() {
        const { prefixCls, className, style, rootComponent: Component, itemRender, loadData, } = this.props;
        if (!itemRender) {
            throw new Error("react-widget-tree-basic: itemRender cannot be empty.");
        }
        if (!loadData) {
            throw new Error("react-widget-tree-basic: loadData cannot be empty.");
        }
        let classes = classNames({
            [prefixCls]: true,
            [className]: className,
        });
        return (React.createElement(TreeContext.Provider, { value: this.getContext() },
            React.createElement(Component, { className: classes, style: style },
                React.createElement(TreeNode, { node: this.getRootNode(), isRoot: true }))));
    }
}
TreeBasic.defaultProps = {
    prefixCls: "rw-tree",
    rootId: null,
    idField: "id",
    leafField: "leaf",
    pidField: "pid",
    maxDepth: 99,
    asyncTestDelay: 16,
    expandedIds: [],
    childrenRender(props) {
        if (!props.expanded)
            return null;
        return props.getChildren();
    },
    rootComponent: "div",
};
export default TreeBasic;
